require 'rubygems'
require 'eventmachine'
require 'nori'

module OtStatus
  VERSION = '0.2.4'

  class Query
    attr_accessor :response

    def initialize connect_timeout=5, inactivity_timeout=10
      @response = nil
      @connect_timeout = connect_timeout
      @inactivity_timeout = inactivity_timeout
    end

    def run hostname, port
      EventMachine.run {
        EventMachine::connect hostname, port, Connection, self, @connect_timeout, @inactivity_timeout
      }

      parse @response
    end

    private
    def parse data
      parser = Nori.new
      data.present? ? parser.parse(data)['tsqp'] : ''
    end

    class Connection < EventMachine::Connection
      QUERY = [6.chr, 0.chr, 255.chr, 255.chr, 'info'].join

      attr_reader :query

      def initialize query, connect_timeout, inactivity_timeout
        @query = query
        self.pending_connect_timeout = connect_timeout
        self.comm_inactivity_timeout = inactivity_timeout
      end

      def post_init
        send_data QUERY
        @data = ""
      end

      def receive_data data
        @data << data
        query.response = @data
      end

      def unbind
        EventMachine::stop_event_loop
      end
    end
  end
end
